package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.utils.Check;
import cn.com.ctop.kuaishou.modules.material.entity.KuaishouAccessToken;
import cn.com.ctop.kuaishou.modules.material.entity.KuaishouResultToken;
import cn.com.ctop.kuaishou.modules.material.service.IKuaishouAccessTokenService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import constant.KuaishouInterfaceConstant;
import org.apache.commons.lang.StringUtils;
import org.jeecg.modules.ctop.entity.AuthTask;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.entity.UserAllocation;
import org.jeecg.modules.ctop.mapper.CTopOauthTokenMapper;
import org.jeecg.modules.ctop.mapper.UserAllocationMapper;
import org.jeecg.modules.ctop.service.IAuthTaskService;
import org.jeecg.modules.ctop.service.IBindAccountAuthService;
import org.jeecg.modules.ctop.service.IUserAllocationService;
import org.jeecg.modules.kuaishou.service.IKuaishouInterfaceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping("/")
public class CallbackController {
    private static final Logger logger = LoggerFactory.getLogger(CallbackController.class);

    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;
    @Autowired
    private IKuaishouAccessTokenService kuaishouAccessTokenService;
    @Autowired
    private CTopOauthTokenMapper cTopOauthTokenMapper;
    @Autowired
    private IBindAccountAuthService bindAccountAuthService;
    @Autowired
    private IAuthTaskService authTaskService;
    @Autowired
    private UserAllocationMapper allocationMapper;
    @Autowired
    private IUserAllocationService allocationService;


    @RequestMapping("/kuaishou")
    @ResponseBody
    public String kuaishou(HttpServletRequest request,
                           HttpServletResponse response,
                           @RequestParam("auth_code") String authCode,
                           @RequestParam("state") String state) {
        try {
            if (StringUtils.isNotBlank(authCode)) {
                KuaishouResultToken accessToken = kuaishouInterfaceService.getAccessToken(authCode);
                KuaishouAccessToken token = accessToken.getData();
                if (token != null) {
                    AuthTask authTask = authTaskService.getAuthTask(state);
                    if (Check.isNull(authTask)) {
                        throw new Exception("根据taskId未取到任务相关信息,taskId:" + state);
                    }
                    String advertiserId = authTask.getAdvertiserId();
                    CTopOauthToken topOauthToken = new CTopOauthToken();
                    Long accountId = token.getAccountId();
                    Map<String, Object> deleteMap = new HashMap<>();
                    deleteMap.put("advertiser_id", advertiserId);
                    deleteMap.put("account_id", accountId);
                    cTopOauthTokenMapper.deleteByMap(deleteMap); //删除 广告主id下的相关授权信息
                    topOauthToken.setAccountId(accountId);
                    topOauthToken.setMediaId("2");
                    topOauthToken.setAdvertiserId(advertiserId);
                    topOauthToken.setAccessToken(token.getAccessToken());
                    long accessTokenExpireIn = token.getAccessTokenExpiresIn() * 1000L;
                    long now = System.currentTimeMillis();
                    Date accessTokenExpireInDate = new Date(now + accessTokenExpireIn);
                    topOauthToken.setAccessTokenExpiresIn(accessTokenExpireInDate);
                    topOauthToken.setRefreshToken(token.getRefreshToken());
                    long refreshTokenExpireIn = token.getRefreshTokenExpiresIn() * 1000L;
                    Date refreshTokenExpireInDate = new Date(now + refreshTokenExpireIn);
                    topOauthToken.setRefreshTokenExpiresIn(refreshTokenExpireInDate);
                    cTopOauthTokenMapper.insert(topOauthToken);
                    bindAccountAuthService.addBindAccount(accountId, KuaishouInterfaceConstant.LOGIN_TYPE_KUAISHOU, advertiserId); //账号绑定


                    UserAllocation userAllocation = allocationService.selectByTaskId(state);
                    userAllocation.setAccountId(accountId);
                    allocationMapper.deleteById(userAllocation.getId());

                    Map<String, Object> deleteUserMap = new HashMap<>();
                    deleteUserMap.put("advertiser_id", advertiserId);
                    deleteUserMap.put("account_id", accountId);
                    allocationMapper.deleteByMap(deleteUserMap);

                    int i = allocationMapper.insert(userAllocation);
                    if (i > 0) {
                        logger.info("同步分配用户表完成,task_id:{},accountId:{}", state, accountId);

                    }
                }
                return "auth_success";
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "auth_fail";
    }
}
